/*
Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.awt.*;
import java.awt.event.*;
import java.util.ResourceBundle;

import ptolemy.plot.Plot;


/**
 * Affichage d'un graphe dans une fentre, avec des valeurs donnes
 */
public class FenetreGraphe extends Frame {
    
    static ResourceBundle rb  = ResourceBundle.getBundle("LocalStrings");
    Label labcoord;
    Plot plot;
    String labelx, labely;
    
    double[] vx;
    double[] vy;
    
    public FenetreGraphe(double[] vx, double[] vy, String labelx, String labely) {
        super("Graphe");
        this.vx = vx;
        this.vy = vy;
        this.labelx = labelx;
        this.labely = labely;
        
        setLayout(new BorderLayout());
        plot = new Plot();
        add(plot, BorderLayout.CENTER);
        plot.setMarksStyle("none");
        plot.setConnected(true);
        if (labelx != null)
            plot.setXLabel(labelx);
        else if (labely != null)
            plot.setXLabel(labely);
        for (int i=0; i<vx.length; i++)
            plot.addPoint(0, vx[i], vy[i], true);
        plot.addMouseMotionListener(new MML());
        
        Panel pbas = new Panel(new GridLayout(2, 1));
        
        labcoord = new Label();
        pbas.add(labcoord);
        
        Panel pfit = new Panel(new FlowLayout());
        Button btableau = new Button(rb.getString("Fermer"));
        btableau.addActionListener(new ActionAdapter() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });
        pfit.add(btableau);
        pbas.add(pfit);
        
        add(pbas, BorderLayout.SOUTH);
        
        plot.setVisible(true);
        pack();
        
        Dimension dim = getSize();
        Dimension ecran = getToolkit().getScreenSize();
        setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        
        validate();
    }
    
    public abstract class ActionAdapter extends Object implements ActionListener {
        public abstract void actionPerformed(ActionEvent ae);
    }
    
    class MML implements MouseMotionListener {
        public void mouseDragged(MouseEvent e) {
        }
        public void mouseMoved(MouseEvent e) {
            double coords[] = plot.getPointCoordinates(e.getX(), e.getY());
            labcoord.setText("x=" + Coupes.joliFormattage(coords[0], 4) +
                "  y=" + Coupes.joliFormattage(coords[1], 4));
            validate();
        }
    }
        
}
