/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var image;
var canvas;
var context;
var canvastrace;
var ctxtrace;
var ctxgrille;
var x1, x2, y1, y2; // coordonnées sous forme de nombres
var labelx, labely; // titres des axes
var wleft = 50; // espace à gauche de l'image
var wright = 10; // à droite
var hbottom = 50; // en bas
var w, h; // dimensions des canvas
var p1, p2; // points de départ et d'arrivée de la ligne à tracer
var drag;
var plot = null; // objet jqplot


// initialisation

function lireParametres() {
    var parametres = new Array();
    var searchString = window.location.search.substring(1);
    var params = searchString.split("&");
    
    for (var i=0; i<params.length; i++) {
        var nomval = params[i].split("=");
        // remarque: les paramètres de l'URL doivent être encodés en UNICODE
        parametres[nomval[0]] = decodeURIComponent(nomval[1]);
    }
    return(parametres);
}

function init() {
    var parametres = lireParametres();
    if (!isCanvasSupported()) {
        // on ne peut pas utiliser excanvas ou flashcanvas pour lire des pixels sur un canvas
        // donc on utilise la version sous forme d'applet en alternative pour les vieux navigateurs
        versionApplet(parametres);
        return;
    }
    var chemin_image = parametres['image'];
    var sx1 = parametres['x1'];
    var sx2 = parametres['x2'];
    var sy1 = parametres['y1'];
    var sy2 = parametres['y2'];
    labelx = parametres['labelx'];
    if (sx1 == null || sx2 == null)
        labelx = null;
    else if (labelx == null)
        labelx = 'x';
    labely = parametres['labely'];
    if (sy1 == null || sy2 == null)
        labely = null;
    else if (labely == null)
        labely = 'y';
    if (chemin_image == null || ((sx1 == null || sx2 == null) && (sy1 == null || sy2 == null))) {
        alert("Paramètres obligatoires: image, x1 et x2 ou y1 et y2; facultatifs: labelx, labely");
        return;
    }
    var chemin_fichiers = parametres['chemin_fichiers'];
    if (chemin_fichiers != null)
        chemin_image = chemin_fichiers + chemin_image;
    x1 = Number(sx1);
    x2 = Number(sx2);
    y1 = Number(sy1);
    y2 = Number(sy2);
    image = new Image();
    image.onload = premierAffichage;
    image.onerror = function(err) { alert("Erreur au chargement de l'image " + chemin_image) };
    image.src = chemin_image;
    p1 = null;
    p2 = null;
    drag = false;
}

function isCanvasSupported() {
    var elem = document.createElement('canvas');
    return !!(elem.getContext && elem.getContext('2d'));
}

function versionApplet(parametres) {
    var affimage = document.getElementById('affimage');
    affimage.style.display = 'none';
    var applet = document.createElement('applet');
    var width, height; // espace disponible pour l'applet
    if (typeof(window.innerWidth) == 'number') {
        // autre que IE
        width = window.innerWidth;
        height = window.innerHeight;
    } else if ( document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight) ) {
        // IE 6+ en mode 'standards compliant'
        width = document.documentElement.clientWidth;
        height = document.documentElement.clientHeight;
    } else {
        width = '100%';
        height = '100%';
    }
    applet.setAttribute('width', width);
    applet.setAttribute('height', height);
    applet.setAttribute('archive', 'Coupes.jar');
    applet.setAttribute('code', 'Coupes');
    var chemin_image = parametres['image'];
    var chemin_fichiers = parametres['chemin_fichiers'];
    if (chemin_fichiers != null)
        chemin_image = chemin_fichiers + chemin_image;
    var param = document.createElement('param');
    param.setAttribute('name', 'image');
    param.setAttribute('value', chemin_image);
    applet.appendChild(param);
    parametreApplet(parametres, 'x1', applet);
    parametreApplet(parametres, 'x2', applet);
    parametreApplet(parametres, 'y1', applet);
    parametreApplet(parametres, 'y2', applet);
    parametreApplet(parametres, 'labelx', applet);
    parametreApplet(parametres, 'labely', applet);
    document.body.appendChild(applet);
}

function parametreApplet(parametres, nom, applet) {
    if (parametres[nom] != null) {
        var param = document.createElement('param');
        param.setAttribute('name', nom);
        param.setAttribute('value', parametres[nom]);
        applet.appendChild(param);
    }
}

function premierAffichage() {
    var divcanvas = document.getElementById('divcanvas');
    canvas = document.createElement('canvas');
    w = image.width + wleft + wright;
    canvas.setAttribute('width', w);
    h = image.height + hbottom;
    canvas.setAttribute('height', h);
    divcanvas.appendChild(canvas);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        G_vmlCanvasManager.initElement(canvas);
    }
    
    if (!canvas.getContext) {
        divcanvas.innerHTML = '<p>Votre navigateur ne gère pas HTML5.</p>';
        return;
    }
    
    context = canvas.getContext('2d');
    
    // les bitmaps offscreen sont impossibles avec excanvas et flashcanvas
    // pour que ça marche avec IE7 et IE8, on utilise 2 autres éléments canvas
    // (un pour la grille et un pour les tracés)
    
    var canvasgrille = document.createElement('canvas');
    canvasgrille.setAttribute('width', w);
    canvasgrille.setAttribute('height', h);
    canvasgrille.style.position = 'absolute';
    canvasgrille.style.left = '0px';
    canvasgrille.style.top = '0px';
    divcanvas.appendChild(canvasgrille);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        canvasgrille = G_vmlCanvasManager.initElement(canvasgrille);
    }
    ctxgrille = canvasgrille.getContext('2d');
    
    canvastrace = document.createElement('canvas');
    canvastrace.setAttribute('width', w);
    canvastrace.setAttribute('height', h);
    canvastrace.style.position = 'absolute';
    canvastrace.style.left = '0px';
    canvastrace.style.top = '0px';
    divcanvas.appendChild(canvastrace);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        canvastrace = G_vmlCanvasManager.initElement(canvastrace);
    }
    ctxtrace = canvastrace.getContext('2d');
    
    if (canvastrace.addEventListener) {
        canvastrace.addEventListener('mousedown', mousedown, false);
        canvastrace.addEventListener('mousemove', mousemove, false);
        canvastrace.addEventListener('mouseup', mouseup, false);
        canvastrace.addEventListener('click', clic, false);
        canvastrace.addEventListener('mouseout', mouseout, false);
    } else {
        canvastrace.attachEvent('onmousedown', mousedown);
        canvastrace.attachEvent('onmousemove', mousemove);
        canvastrace.attachEvent('onmouseup', mouseup);
        canvastrace.attachEvent('onclick', clic);
        canvastrace.attachEvent('onmouseout', mouseout);
    }
    if ('ontouchstart' in document) {
        canvastrace.addEventListener('touchstart', mousedown, false);
        canvastrace.addEventListener('touchmove', mousemove, false);
        canvastrace.addEventListener('touchend', mouseup, false);
    }
    
    if (document.addEventListener) {
        document.addEventListener('keydown', keydown, false);
        document.addEventListener('keypress', keypress, false);
    } else if (document.attachEvent) {
        document.attachEvent('onkeydown', keydown);
        document.attachEvent('onkeypress', keypress);
    }
    
    context.drawImage(image, wleft, 0);
    grille(ctxgrille);
}


// évènements

function coordonnees(e) {
    // résultat décalé de 1px avec excanvas :(
    var x, y;
    if (e.pageX || e.pageY) {
      x = e.pageX;
      y = e.pageY;
    } else {
      x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
      y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    }
    var totalOffsetX = 0;
    var totalOffsetY = 0;
    var currentElement = canvastrace;
    do {
        totalOffsetX += currentElement.offsetLeft;
        totalOffsetY += currentElement.offsetTop;
    } while (currentElement = currentElement.offsetParent)
    x -= totalOffsetX;
    y -= totalOffsetY;
    return( { x : x, y : y } );
}

function mousedown(e) {
    effacerTrace();
    var pt = coordonnees(e);
    if (pt.x < wleft || pt.x >= wleft + image.width || pt.y >= image.height)
        return;
    if (p2 != null)
        p2 = null;
    p1 = pt;
    drag = true;
    if (e.preventDefault)
        e.preventDefault();
    else
        e.returnValue = false;
}

function mousemove(e) {
    var pt = coordonnees(e);
    if (pt.x < wleft)
        pt.x = wleft;
    if (pt.x >= wleft + image.width)
        pt.x = wleft + image.width - 1;
    if (pt.y >= image.height)
        pt.y = image.height - 1;
    var x, y, sx, sy;
    if (labelx != null) {
        x = ((pt.x - wleft) * 1.0 / image.width) * (x2 - x1) + x1;
        sx = joliFormattage(x, 5);
    }
    if (labely != null) {
        y = (pt.y * 1.0 / (h - hbottom)) * (y2 - y1) + y1;
        sy = joliFormattage(y, 5);
    }
    var spancoord = document.getElementById('coordonnees');
    if (labelx != null && labely != null)
        spancoord.innerHTML = sx + ', ' + sy;
    else if (labelx != null)
        spancoord.innerHTML = sx;
    else if (labely != null)
        spancoord.innerHTML = sy;
    if (drag) {
        if (p1 != null) {
            var nouveau = (p2 == null);
            if (!nouveau)
                effacerTrace();
            p2 = pt;
            dessinerLigne(ctxtrace);
            if (nouveau)
                document.getElementById('bouton').disabled = false;
        }
    }
}

function mouseup(e) {
    drag = false;
}

function clic(e) {
    mousemove(e);
}

function mouseout(e) {
    if (drag)
        mouseup(e);
}

function keydown(e) {
    switch (e.keyCode) {
        case 37: deplacer(-1, 0); break; // left
        case 39: deplacer(1, 0); break; // right
        case 38: deplacer(0, -1); break; // up
        case 40: deplacer(0, 1); break; // down
        default: ;
    }
}

function keypress(e) {
    if (e.ctrlKey || e.metaKey)
        return;
    var code;
    if (e.which == null)
        code = e.keyCode;    // old IE
    else if (e.which != 0 && e.charCode != 0)
        code = e.which;	  // All others
    else
        return; // special key
    var c = String.fromCharCode(code);
    switch (c) {
        case '+': agrandir(1); break;
        case '-': agrandir(-1); break;
        default: ;
    }
}

function deplacer(dx, dy) {
    effacerTrace();
    if (p1 != null) {
        p1.x += dx;
        p1.y += dy;
    }
    if (p2 != null) {
        p2.x += dx;
        p2.y += dy;
    }
    dessinerLigne(ctxtrace);
}


// tracés

function effacerTrace() {
    ctxtrace.clearRect(wleft, 0, image.width, image.height);
}

function dessinerLigne(ctx) {
    ctx.beginPath();
    ctx.strokeStyle = 'black';
    ctx.moveTo(p1.x + 0.5, p1.y + 0.5);
    ctx.lineTo(p2.x + 0.5, p2.y + 0.5);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = 'white';
    ctx.moveTo(p1.x - 0.5, p1.y - 0.5);
    ctx.lineTo(p2.x - 0.5, p2.y - 0.5);
    ctx.stroke();
}


// affichage de la grille

function grille(ctx) {
    ctx.font = 'normal 10px sans-serif'; // il faut ajouter normal pour éviter un bug de FlashCanvas
    ctx.textAlign = 'left';
    ctx.textBaseline = 'alphabetic';
    ctx.fillStyle = 'black';
    
    // Axe x
    
    if (labelx != null) {
        var ndiv = Math.floor(image.width/30);
        if (ndiv > 15)
            ndiv = 15;
        var fx = ndiv / Math.abs(x2 - x1);
        var exposant_fx;
        if (fx != 0)
            exposant_fx = Math.floor(Math.log(Math.abs(fx))/Math.log(10));
        else
            exposant_fx = 0;
        var mantisse_fx = fx / (Math.pow(10,exposant_fx));
        mantisse_fx = Math.round(mantisse_fx);
        // on guarde les chiffres dont l'inverse n'a pas trop de décimales
        if (mantisse_fx == 3)
            mantisse_fx = 2;
        else if (mantisse_fx == 6)
            mantisse_fx = 5;
        else if (mantisse_fx > 6)
            mantisse_fx = 10;
        fx = mantisse_fx * Math.pow(10, exposant_fx);
        
        grilleX(ctx, fx);
        
        ctx.font = 'bold 10px sans-serif';
        var metrics = ctx.measureText(labelx);
        ctx.fillText(labelx, wleft + image.width/2 - metrics.width/2, h - hbottom/4);
        ctx.font = 'normal 10px sans-serif';
    }
    
    // Axe y
    
    if (labely != null) {
        var ndiv = Math.floor(image.height/15);
        if (ndiv > 15)
            ndiv = 15;
        var fy = ndiv / Math.abs(y2 - y1);
        var exposant_fy;
        if (fy != 0)
            exposant_fy = Math.floor(Math.log(Math.abs(fy))/Math.log(10));
        else
            exposant_fy = 0;
        var mantisse_fy = fy / (Math.pow(10,exposant_fy));
        mantisse_fy = Math.round(mantisse_fy);
        if (mantisse_fy == 3)
            mantisse_fy = 4;
        else if (mantisse_fy == 6)
            mantisse_fy = 5;
        else if (mantisse_fy > 6)
            mantisse_fy = 10;
        fy = mantisse_fy * Math.pow(10, exposant_fy);
        grilleY(ctx, fy);
        
        ctx.save();
        ctx.font = 'bold 10px sans-serif';
        ctx.translate(10, (h - hbottom)/2);
        ctx.rotate(-Math.PI/2);
        ctx.textAlign = 'center';
        ctx.fillText(labely, 0, 0);
        ctx.restore();
    }
    
}

function grilleX(ctx, facteur) {
    var ix1, ix2, px;
    var scoord;
    
    ctx.save();
    if (x2 > x1) {
        ix1 = Math.ceil(x1*facteur);
        ix2 = Math.floor(x2*facteur);
    } else {
        ix1 = Math.ceil(x2*facteur);
        ix2 = Math.floor(x1*facteur);
    }
    for (var i=ix1; i<=ix2; i++) {
        if (x2 > x1)
            px = Math.round(((i/facteur - x1) / (x2 - x1)) * image.width) + wleft;
        else
            px = wleft + image.width - Math.round(((i/facteur - x2) / (x1 - x2)) * image.width);
        scoord = joliFormattage(i/facteur, 3);
        var metrics = ctx.measureText(scoord);
        ctx.fillText(scoord, px - metrics.width/2, h - hbottom/2);
        ctx.beginPath();
        ctx.strokeStyle = 'white';
        ctx.moveTo(px-0.5, 0);
        ctx.lineTo(px-0.5, h - hbottom);
        ctx.stroke();
        ctx.beginPath();
        ctx.strokeStyle = 'black';
        ctx.moveTo(px+0.5, 0);
        ctx.lineTo(px+0.5, h - hbottom);
        ctx.stroke();
    }
    ctx.restore();
}

function grilleY(ctx, facteur) {
    var iy1, iy2, py;
    var scoord;
    
    ctx.save();
    if (y2 > y1) {
        iy1 = Math.ceil(y1*facteur);
        iy2 = Math.floor(y2*facteur);
    } else {
        iy1 = Math.ceil(y2*facteur);
        iy2 = Math.floor(y1*facteur);
    }
    for (var i=iy1; i<=iy2; i++) {
        if (y2 > y1)
            py = Math.round(((i/facteur - y1) / (y2 - y1)) * image.height);
        else
            py = image.height - Math.round(((i/facteur - y2) / (y1 - y2)) * image.height);
        scoord = joliFormattage(i/facteur, 3);
        ctx.fillText(scoord, 12, py + 2);
        ctx.beginPath();
        ctx.strokeStyle = 'white';
        ctx.moveTo(wleft, py-0.5);
        ctx.lineTo(wleft + image.width, py-0.5);
        ctx.stroke();
        ctx.beginPath();
        ctx.strokeStyle = 'black';
        ctx.moveTo(wleft, py+0.5);
        ctx.lineTo(wleft + image.width, py+0.5);
        ctx.stroke();
    }
    ctx.restore();
}


// graphe

function tracer() {
    if (p1 == null || p2 == null) {
        alert("Il faut tracer une coupe sur l'image avant de pouvoir afficher le graphe correspondant.");
        return;
    }
    var affimage = document.getElementById('affimage');
    affimage.style.display = 'none';
    var divgraphe = document.getElementById('affgraphe');
    divgraphe.style.display = 'block';
    var ff;
    if (labely == null)
        ff = (x2 - x1) / image.width;
    else if (labelx == null)
        ff = (y2 - y1) / image.height;
    else
        ff = (((x2 - x1) / image.width) + ((y2 - y1) / image.height)) / 2;
    var imageData = context.getImageData(wleft, 0, image.width, image.height); // marche pas avec excanvas ?
    var lx1 = p1.x - wleft;
    var lx2 = p2.x - wleft;
    var ly1 = p1.y;
    var ly2 = p2.y;
    var dx = Math.abs(lx2 - lx1);
    var dy = Math.abs(ly2 - ly1);
    var vx;
    var vy;
    var x, y;
    if (dx > dy) {
        vx = new Array(dx);
        vy = new Array(dx);
        var sgn;
        if (lx2 > lx1)
            sgn = 1;
        else
            sgn = -1;
        for (var i=0; i<dx; i++) {
            x = lx1 + sgn * i;
            y = ly1 + Math.round((ly2 - ly1) * (i / dx));
            if (labely == null)
                vx[i] = x1 + x * ff;
            else if (labelx == null)
                vx[i] = y1 + y * ff;
            else
                vx[i] = Math.sqrt((x - lx1)*(x - lx1) + (y - ly1)*(y - ly1)) * ff;
            var offset = 4 * (y * image.width + x)
            var red = imageData.data[offset];
            var green = imageData.data[offset + 1];
            var blue = imageData.data[offset + 2];
            vy[i] = (red + green + blue) / 3;
        }
    } else {
        vx = new Array(dy);
        vy = new Array(dy);
        var sgn;
        if (ly2 > ly1)
            sgn = 1;
        else
            sgn = -1;
        for (var i=0; i<dy; i++) {
            x = lx1 + Math.round((lx2 - lx1) * (i / dy));
            y = ly1 + sgn * i;
            if (labely == null)
                vx[i] = x1 + x * ff;
            else if (labelx == null)
                vx[i] = y1 + y * ff;
            else
                vx[i] = Math.sqrt((x - lx1)*(x - lx1) + (y - ly1)*(y - ly1)) * ff;
            var offset = 4 * (y * image.width + x)
            var red = imageData.data[offset];
            var green = imageData.data[offset + 1];
            var blue = imageData.data[offset + 2];
            vy[i] = (red + green + blue) / 3;
        }
    }
    affGraphe(vx, vy);
}

function retour() {
    var divgraphe = document.getElementById('affgraphe');
    divgraphe.style.display = 'none';
    var affimage = document.getElementById('affimage');
    affimage.style.display = 'block';
}

function affGraphe(vx, vy) {
    var points = [];
    for (var i=0; i<vx.length; i++)
        if (vx[i] != null && vy[i] != null && !isNaN(vx[i]) && !isNaN(vy[i]))
            points.push([vx[i], vy[i]]);
    if (plot != null)
        $('#plot').empty();
    plot = $.jqplot ('plot', [points], {
        axesDefaults: {
            pad: 1.1,
            labelRenderer: $.jqplot.CanvasAxisLabelRenderer
        },
        axes: {
            xaxis: {
                label: labelx
            },
            yaxis: {
                label: 'intensité'
            }
        },
        seriesDefaults: {
            lineWidth: 1.5,
            shadow: false
        },
        series : [
            {
                showLine: true,
                markerOptions: { show: false }
            }
        ],
        cursor: {
            show: true,
            zoom: true,
            showTooltip: true
        },
        highlighter: {
            show: true,
            useAxesFormatters: false
        }
    });
}


// fonction issue du Calcotron
function joliFormattage(v, precision) {
    var limitesci = 3; // décimales avant de passer en mode sci
    
    if (v == 0)
        return('0');
    var sj = ''+v;
    if ('Infinity' === sj || '-Infinity' === sj || 'NaN' === sj)
        return(''+v);
    var signe;
    if (v >= 0)
        signe = '';
    else {
        v = -v;
        signe = '-';
    }
    var exposant = Math.floor(Math.log(v)/Math.log(10));
    var exparr = Math.pow(10, precision);
    var mantisse = v / Math.pow(10, exposant);
    mantisse = Math.round(mantisse*exparr)/exparr;
    if (mantisse >= 10) {
        exposant += 1;
        mantisse = v / Math.pow(10, exposant);
        mantisse = Math.round(mantisse*exparr)/exparr;
    }
    var sv = '';
    var chiffre = Math.floor(mantisse);
    sv += chiffre + '.';
    for (var i=0; i<precision-1; i++) {
        mantisse = (mantisse - chiffre)*10;
        var exparr2 = Math.pow(10, precision-i);
        mantisse = (Math.round(mantisse*exparr))/exparr;
        chiffre = Math.floor(mantisse);
        sv += chiffre;
    }
    mantisse = (mantisse - chiffre)*10;
    chiffre = Math.round(mantisse);
    sv += chiffre;
    var dernier = sv.charAt(sv.length-1);
    while (dernier == '0') {
        sv = sv.substring(0, sv.length-1);
        dernier = sv.charAt(sv.length-1);
    }
    if (exposant != 0) {
        if (exposant > 0 && exposant < limitesci) {
            var sd = sv.substring(0,1);
            if (sv.length-2 < exposant) {
                // transformation de 3.2E2 en 320
                if (sv.length > 2)
                    sd += sv.substring(2,sv.length);
                for (var i=0; i<exposant-(sv.length-2); i++)
                    sd += '0';
            } else {
                // transformation de 3.245E2 en 324.5
                if (sv.length < exposant+2)
                    sd += sv.substring(2);
                else
                    sd += sv.substring(2,exposant+2) + '.' + sv.substring(exposant+2);
            }
            sv = sd;
        } else if (exposant < 0 && exposant > -limitesci) {
            // transformation de 3.2E-2 en 0.032
            var sd = '0.';
            for (var i=0; i<-exposant-1; i++)
                sd += '0';
            sv = sd + sv.charAt(0) + sv.substring(2);
        } else
            sv += 'E' + exposant;
    }
    if (sv.charAt(sv.length-1) == '.')
        sv = sv.substring(0, sv.length-1);
    
    sv = signe + sv;
    
    return(sv);
}

